module.exports = {
  cname: "kitware.github.io",
  baseUrl: "/wslink",
  work: "./build-tmp",
  // api: ['../js/src'],
  examples: [],
  config: {
    title: "Wslink",
    description: '"JavaScript and Python communication"',
    subtitle: '"Enable scientific visualization to the Web."',
    author: "Kitware Inc.",
    timezone: "UTC",
    url: "https://kitware.github.io/wslink",
    root: "/wslink/",
    github: "kitware/wslink",
    // google_analytics: 'UA-90338862-5',
  },
  copy: [],
};
