.container[data-v-3d210c8b] {
  width: 400px;
}
.line[data-v-3d210c8b] {
  display: flex;
  justify-content: space-between;
  padding: 5px 0;
}
.button[data-v-3d210c8b] {
  width: 100px;
}
.input[data-v-3d210c8b] {
  flex: 1;
  margin-right: 10px;
}
.textarea[data-v-3d210c8b] {
  width: 390px;
  display: block;
  resize: none;
}
