// .prettierrc.js
module.exports = {
  trailingComma: "es5", // add trailing commas to objects, arrays, etc.
  tabWidth: 2,
  semi: true, // add semicolons at the end of statements
  singleQuote: false, // use single quotes instead of double quotes
  printWidth: 80, // specify the line length that the printer will wrap on
};
