def test_import_root():
    import wslink  # noqa: F401


def test_import_chunking():
    import wslink.chunking  # noqa: F401


def test_import_emitter():
    import wslink.emitter  # noqa: F401


def test_import_launcher():
    import wslink.launcher  # noqa: F401


def test_import_protocol():
    import wslink.protocol  # noqa: F401


def test_import_publish():
    import wslink.publish  # noqa: F401


def test_import_relay():
    import wslink.relay  # noqa: F401


def test_import_server():
    import wslink.server  # noqa: F401


def test_import_ssl():
    import wslink.ssl_context  # noqa: F401


def test_import_uri():
    import wslink.uri  # noqa: F401


def test_import_websocket():
    import wslink.websocket  # noqa: F401


def test_import_backends():
    import wslink.backends  # noqa: F401


def test_import_backends_aiohttp():
    import wslink.backends.aiohttp  # noqa: F401


def test_import_backends_generic():
    import wslink.backends.generic  # noqa: F401
